# https://typst.app/docs/
#

# Detection
# ‾‾‾‾‾‾‾‾‾

hook global BufCreate .*[.](typ|typst) %{
	set-option buffer filetype typst
	set-option buffer comment_line "//"
	set-option buffer formatcmd "typstyle --wrap-text"
}

# Initialization
# ‾‾‾‾‾‾‾‾‾‾‾‾‾‾

hook global WinSetOption filetype=typst %(
	require-module typst

    hook window ModeChange pop:insert:.* -group typst-trim-indent typst-trim-indent
    hook window InsertChar .* -group typst-indent typst-indent-on-char
    hook window InsertChar \n -group typst-indent typst-indent-on-new-line

    hook -once -always window WinSetOption filetype=.* %{ remove-hooks window typst-.+ }
)

hook -group typst-highlight global WinSetOption filetype=typst %{
	add-highlighter window/typst ref typst
	hook -once -always window WinSetOption filetype=.* %{ remove-highlighter window/typst }
}

provide-module typst %~

# not setting highlights b/c it's handled by treesitter

# Indent
# ------

define-command -hidden typst-trim-indent %{
    # remove trailing white spaces
    try %{ execute-keys -draft -itersel x s \h+$ <ret> d }
}

define-command -hidden typst-indent-on-char %<
    evaluate-commands -draft -itersel %<
        # align closer token to its opener when alone on a line
        try %< execute-keys -draft <a-h> <a-k> ^\h+[}\])]$ <ret> m <a-S> 1<a-&> >
    >
>

define-command -hidden typst-indent-on-new-line %<
    evaluate-commands -draft -itersel %<
        # preserve previous line indent
        try %{ execute-keys -draft <semicolon> K <a-&> }
        # filter previous line
        try %{ execute-keys -draft k : typst-trim-indent <ret> }
        # indent after lines ending with opener token
        try %< execute-keys -draft k x <a-k> [({[]\h*$ <ret> j <a-gt> >
        # deindent closer token(s) when after cursor
        try %< execute-keys -draft x <a-k> ^\h*[}\])] <ret> gh / [}\])] <ret> m <a-S> 1<a-&> >
    >
>

~
